"""ZPhotoSlides Folder
"""
from Globals import Persistent
from Globals import package_home
from OFS.ObjectManager import ObjectManager
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from OFS.FindSupport import FindSupport
from AccessControl.Role import RoleManager
from AccessControl import ClassSecurityInfo
from Globals import MessageDialog
from Globals import InitializeClass, DTMLFile,HTMLFile
from ZPhotoSlides import ZPhotoSlides
from Products.ZCatalog.ZCatalog import ZCatalog
import os
import string

try:
	from Products.Localizer import LocalDTMLFile
	from Products.Localizer import Gettext
	gettext = Gettext.translation(globals())
except:
	have_localizer = 0
	from Globals import DTMLFile
	LocalDTMLFile = DTMLFile
	del DTMLFile
else:
	have_localizer = 1

if(not have_localizer):
	from utils import gettext
_ = gettext

def upload_file(path,_prefix=None):
        if _prefix is None: _prefix=SOFTWARE_HOME
        elif type(_prefix) is not type(''):
            _prefix=package_home(_prefix)
        path = os.path.join(_prefix, path)

        file=open(path, 'rb')
        data=file.read()
        file.close()
        return data

class ZPhotoSlidesFolder(
                         ObjectManager,
                         PropertyManager,
                         RoleManager,
                         Item,
                         FindSupport):
    """ZPhotoSlides Folder object implementation."""
    if(have_localizer):
    	locale = package_home(globals()) + '/locale/'
    	_gettext = gettext

    manage_options = (
                       {'label': "Manage ZPhotoSlides",    'action':'manage_main'},
		       {'label': "View",	'action': 'index_html'},
	               {'label': "Properties",       'action': 'manage_props'},
                       {'label': "Edit layout", 'action': 'user_layout_folder'},
		       {'label': "Manage positions",'action': 'manage_position'},
		       {'label': "Search for photos",'action': 'search_photos'},
		       {'label': "Statistics",'action': 'stat_folder'},
                       {'label': "Security",   'action': 'manage_access'},
                       {'label': "Undo", 'action': 'manage_UndoForm'},
                       {'label': "Ownership", 'action': 'manage_owner'},
		       {'label': "About this product", 'action':'about'}
		     )
 
    meta_type = "ZPhotoSlides Folder"
    index_html = LocalDTMLFile('dtml/index_folder_html',globals(),gettext=gettext)
    manage_position = LocalDTMLFile('dtml/manage_ZPS_position',globals(),gettext=gettext)
    manage_tabs = LocalDTMLFile('dtml/manage_tabs',globals(),gettext=gettext)
    manage_props = LocalDTMLFile('dtml/ZPhotoSlidesFolderEdit',globals(),gettext=gettext) # edit main properties
    user_layout_folder = LocalDTMLFile('dtml/user_layout_folder',globals(),gettext=gettext)
    about = LocalDTMLFile('dtml/about',globals(),gettext=gettext)                         # about us
    css_file = HTMLFile('dtml/stylesheet_demo.css',globals(),gettext=gettext) 
       # main css file
    search_photos = LocalDTMLFile('dtml/search_photo',globals(),gettext=gettext)
    search_result = LocalDTMLFile('dtml/search_result',globals(),gettext=gettext)
    manage_main = LocalDTMLFile('dtml/list_zphotoslides',globals(),gettext=gettext)
    stat_folder = LocalDTMLFile('dtml/stat_folder',globals(),gettext=gettext)
    
    _properties = (
        {'id':'title', 'type': 'string', 'mode': 'w'},
        )

    security=ClassSecurityInfo()

    def __init__(self, id, title, description,position):

        self.version = '0.4'
        self.id = id
        self.title = title
	self.position = position
        self.description = description
        self.actual_css = 'stylesheet_base_css'
        self.update_date = '21/02/2003'
        self.zphoto_header_folder= ''                  # get content of standard header Folder
        self.zphoto_footer_folder= ''                  # get content of standard header Folder
        self.keywords = ''                      # keywords describing the object
        self.description = ''                   # description of the object
        self.manage_addFolder('css', 'Thumbnails', createPublic=0, createUserF=0, REQUEST=None)
	catalog = ZCatalog('catalog',container=self)
	self._setObject('catalog', catalog)
	self.manage_catalog()
	self.css._setProperty('amount_css',1,'int')
	self.css.manage_addDTMLMethod('stylesheet_base_css', 'Main ZPhotoSlidesFolder Stylesheet', upload_file('dtml/stylesheet_demo.css.dtml', globals()), REQUEST=None)

    def manage_catalog(self):
	try:    #for 2.5 ZCatalog which add automaticaly id and title and bobobase_modification_time
		self.catalog.manage_delIndex(ids=['id','title','bobobase_modification_time'])
	except:
		pass
	self.catalog.addIndex('id','FieldIndex')
	self.catalog.addIndex('title','TextIndex')
	self.catalog.addIndex('keywords','TextIndex')
	try:
		self.catalog.addIndex('bobobase_modification_time','DateIndex')
	except:	#zope 2.5 has no DateIndex =(
		self.catalog.addIndex('bobobase_modification_time','FieldIndex')
 
    security.declareProtected('View management screens','change_order_by_date')
    def reCatalog(self, REQUEST=None):
	"""clear the catalog, and put all images into it."""
        self.catalog._catalog.clear()
        path='/'+string.split(REQUEST.URL1, REQUEST.SERVER_URL)[1][1:]
	self.catalog.ZopeFindAndApply(self, obj_metatypes=['Image'], search_sub=1,REQUEST=REQUEST, apply_func=self.catalog.catalog_object, apply_path=path)
	return self.search_photos(manage_tabs_message="Objects recataloged!")

   # def update_catalog_index(self):
   #	"update the index of the catalog"
   #	self.catalog.manage_reindexIndex(['id','title'])

    def upgrade_zphotoslides(self):
	"upgrade all ZPhotoSlides inside"
	for zphoto in self.objectValues('ZPhotoSlides'):
		zphoto.position = 0
		try:
			zphoto.upgrade()
		except:
			continue
	return self.manage_props(manage_tabs_message=_('ZPhotoSlides Updated'))

    security.declareProtected('View management screens','change_order_by_date')
    def change_order_by_date(self,REQUEST):
	"update position of the different zphotoslides and zps fallowing the date of modification"
	list_zp = self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder'])
	list_zp.sort(lambda x, y: cmp(x.bobobase_modification_time,y.bobobase_modification_time))
	cpt = 1
	for item in list_zp:
		item.position = cpt
		cpt = cpt + 1
	return self.manage_position(manage_tabs_message=_("Positions updated!"))

    security.declareProtected('View management screens', 'change_order')
    def change_order(self,REQUEST):
	"change the order of the zphotoslides and zps"
 	for zphoto_item in self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder']):
		item = zphoto_item.getId() + '_position'
		for key in REQUEST.form.keys():
			if(item == key):
				zphoto_item.position = int(REQUEST.form[key])
	return self.manage_position(manage_tabs_message="Positions updated!",gettext=gettext)

    security.declareProtected('View management screens','needupgrade')
    def needupgrade(self,REQUEST=None):
	if(self.version in ['0.2','0.3']):
		return 1
	else:
		return 0

    security.declareProtected('View management screens', 'upgrade')
    def upgrade(self,REQUEST):
	"Allow the user to Upgrade to the new ZPhotoSlides Folder"
	if(self.version == '0.2'):
		self.zphoto_header_folder= ''
		self.zphoto_footer_folder= ''
		try:
			del self.index_html
		except:
			pass
		self.keywords = ''
		self.version = '0.3'
	if(self.version == '0.3'):
		#add catalog
		try:
			catalog = ZCatalog('catalog',container=self)
			self._setObject('catalog',catalog)
			self.manage_catalog()
			self.reCatalog(REQUEST)
		except:
			pass
		self.position = len(self.aq_parent.objectValues(['ZPhotoSlides','ZPhotoSlides Folder']))
		
		self.version = '0.4' 
	return self.manage_props(REQUEST,manage_tabs_message='ZPhotoSlides Folder Upgraded!')

    security.declareProtected('Access contents information', 'zphotoslidesIds')
    def manage_edit(self, title,description,REQUEST):
        """Edit the main properties..."""
        self.title = title
        self.description = description
        if REQUEST is not None:
            return MessageDialog(
                title = _("Edited"),
                message = _("Properties for") + " %s " % self.id + _("changed"),
                action = './manage_props',
                )

    security.declareProtected('Access contents information', 'zphotoslidesIds')
    def zphotoslidesIds(self):
        """Return list of ZPhotoSlides in this folder."""
        return self.objectIds(['ZPhotoSlides'])

    def image_select(self):
        return [''] + self.photoIds()

    def get_all_photos(self):
	list = []
	for zphoto in self.objectValues('ZPhotoSlides'):
		list = list + zphoto.thumbnails.objectValues('Image')
	for folder in self.objectValues('ZPhotoSlides Folder'):
		list = list + folder.get_all_photos()
	return list

    security.declareProtected('Access contents information', 'numZPhotoSlides')
    def numZPhotoSlides(self):
        """Return number of ZPhotoSlides in folder tree."""
        ZPhotoSlides = len(self.objectIds(['ZPhotoSlides']))
        for folder in self.objectValues(['ZPhotoSlides Folder']):
            ZPhotoSlides = ZPhotoSlides + folder.numZPhotoSlides()
        return ZPhotoSlides

    def numZPhotoSlidesFolder(self):
	nbr = len(self.objectIds(['ZPhotoSlides Folder']))
	for folder in self.objectValues(['ZPhotoSlides Folder']):
            nbr = nbr + folder.numZPhotoSlidesFolder()
	return nbr

    def get_nbr_non_hidden_thumb(self):
	"return the number of photos inside all the ZPhotoSlides of the folder"
	sum = 0
	for zphoto in self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder']):
		res = zphoto.get_nbr_non_hidden_thumb()
		if(type(res) == type([])):
			sum = sum + len(zphoto.get_nbr_non_hidden_thumb())
		else:
			sum = sum + zphoto.get_nbr_non_hidden_thumb()
	return sum

    def give_header_4_robots(self):
	"give the header for the robots"
	return """
<meta name="KEYWORDS" content="%s" />
<meta name="DESCRIPTION" content="%s" />
<meta name="resource-type" content="document" />
<meta name="ROBOTS" content="INDEX,FOLLOW" />
<meta http-equiv="expires" content="0" />
<meta name="author" content="ZPhotoSlides Folder" />
<meta name="copyright" content="ZPhotoSlides Folder %s" />
<meta name="revisit-after" content="1 days" />
<meta name="rating" content="General" />""" % (self.keywords,self.description,self.version)

    security.declareProtected('View management screens','new_header_footer_folder')
    def new_header_footer_folder(self,REQUEST):
	"Change the standard_html_header and footer Folder"
	self.zphoto_header_folder = REQUEST.header
	self.zphoto_footer_folder = REQUEST.footer
	if(REQUEST.header != '' or REQUEST.footer != ''):
                msg1 = _("""You use yet the standard html footer and header. To use the css files again
                make the standard html footer and header <u><i>EMPTY</i></u> in the layout formular!""")
                title_1 = "Using standard html header and footer"
        else:
		msg1 = _("You use yet the CSS file.")
		title_1 = _("Using CSS file")
	return self.REQUEST.RESPONSE.redirect('%s/user_layout_folder?message1=%s&message_title=%s' % (REQUEST.URL1, msg1, title_1))

    security.declareProtected('Access contents information', 'nextZPhotoSlidesFolder')
    def nextZPhotoSlidesFolder(self):
        """Return next ZPhotoSlides Folder."""
        id = self.getId()
        folderIds = self.aq_parent.objectIds(['ZPhotoSlides Folder'])
        folderIds.sort()
        if id == folderIds[-1]:
            return None
        return getattr(self.aq_parent, folderIds[folderIds.index(id)+1])

    security.declareProtected('Access contents information', 'prevZPhotoSlidesFolder')
    def prevZPhotoSlidesFolder(self):
        """Return previous ZPhotoSlides Folder."""
        id = self.getId()
        folderIds = self.aq_parent.objectIds(['ZPhotoSlides Folder'])
        folderIds.sort()
        if id == folderIds[0]:
            return None
        return getattr(self.aq_parent, folderIds[folderIds.index(id)-1])

    def __len__(self):
	return len(self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder']))

    def __getitem__(self,i):
	list = self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder'])
	list.sort(lambda x,y: cmp(x.position,y.position))
	return list[i]

    def index(self,item):
	list = self.objectValues(['ZPhotoSlides','ZPhotoSlides Folder'])
	list.sort(lambda x,y: cmp(x.position,y.position))
	return list.index(item)

manage_addZPhotoSlidesFolderForm = LocalDTMLFile('dtml/addZPhotoSlidesFolderForm', globals(),gettext=gettext)

def manage_addZPhotoSlidesFolder(dispatcher, id, title,description, REQUEST=None):
    """Add ZPhotoSlides Folder object"""
    dest = dispatcher.Destination()
    position = len(dispatcher.objectValues(['ZPhotoSlides Folder','ZPhotoSlides'])) + 1
    dest._setObject(id, ZPhotoSlidesFolder(id, title,description,position))
    self = dest._getOb(id)

    if REQUEST is not None:
        try:    url=dispatcher.DestinationURL()
        except: url=REQUEST['URL1']
        REQUEST.RESPONSE.redirect('%s/%s/manage_props' % (url,id))
    return id

InitializeClass(ZPhotoSlidesFolder)
